import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:pam_flutter/models/consent_message_model.dart';
import 'package:pam_flutter/repositories/base_repository.dart';
import 'package:pam_flutter/repositories/pdpa_repository.dart';
import 'package:pam_flutter/utils/time_helper.dart';
import 'package:pam_flutter/widgets/commons/loading_stack.dart';
import 'package:pam_flutter/widgets/commons/primary_button.dart';
import 'package:pam_flutter/widgets/consent_item.dart';
import 'package:pam_flutter/widgets/consent_language.dart';

class ConsentPage extends StatefulWidget {
  final String consentId;
  final String baseAPI;

  ConsentPage({
    required this.consentId,
    required this.baseAPI,
  });

  @override
  State<StatefulWidget> createState() {
    return _ConsentPageState();
  }
}

class _ConsentPageState extends State<ConsentPage> {
  late String language;
  late PDPARepository pdpaRepository;

  @override
  void initState() {
    super.initState();

    this.language = "th";
    this.pdpaRepository = PDPARepository(
      buildCtx: this.context,
      options: NewRepositoryOptions(
        baseUrl: "${widget.baseAPI}/api/pdpa/consent-message",
        findUrl: "${widget.baseAPI}/api/pdpa/consent-message/:id",
      ),
    );
    this.pdpaRepository.get(widget.consentId).then((_) {
      this.language = this.pdpaRepository.data?.Setting.DefaultLanguage ?? "th";
    });
  }

  @override
  void dispose() {
    super.dispose();

    this.pdpaRepository.dispose();
  }

  void Function(bool isAllow) onConsentAllowedChanged(String settingName) {
    return (bool isAllowed) {
      return this.pdpaRepository.allowConsentSetting(settingName, isAllowed);
    };
  }

  @override
  Widget build(BuildContext context) {
    bool isTablet = MediaQuery.of(context).size.width >= 768;

    return Container(
      alignment: Alignment.center,
      child: LoadingStack(
        isLoadingSC: this.pdpaRepository.isLoadingSC,
        children: () => [
          Column(
            children: [
              Container(
                height: MediaQuery.of(context).padding.top,
              ),
              Container(
                padding: EdgeInsets.all(16),
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: [
                    Expanded(
                      child: Container(
                        child: Text(
                          this.pdpaRepository.data?.Name ?? "",
                          overflow: TextOverflow.ellipsis,
                          textAlign: TextAlign.start,
                          style: TextStyle(
                            color: this
                                .pdpaRepository
                                .data
                                ?.StyleConfiguration
                                .ConsentDetail
                                .PrimaryColor,
                            fontSize: 32,
                            fontWeight: FontWeight.bold,
                          ),
                        ),
                      ),
                    ),
                    Container(
                      padding: EdgeInsets.only(
                        left: !isTablet ? 0 : 16,
                        right: !isTablet ? 0 : 16,
                      ),
                      child: Text(
                        !isTablet
                            ? ""
                            : this.pdpaRepository.data?.Description ?? "",
                        overflow: TextOverflow.ellipsis,
                        textAlign: TextAlign.start,
                        style: TextStyle(fontSize: 18),
                      ),
                    ),
                    Container(
                      margin: EdgeInsets.only(
                        left: 16,
                      ),
                      child: Row(
                        children: List.generate(
                            this
                                    .pdpaRepository
                                    .data
                                    ?.Setting
                                    .AvailableLanguages
                                    .length ??
                                0, (index) {
                          String? language = this.pdpaRepository.data?.Setting.AvailableLanguages[index];
                          bool isActive = this.language == language;
                          ConsentMessageStyleConfigurationConsentDetailModel?
                              style = this
                                  .pdpaRepository
                                  .data
                                  ?.StyleConfiguration
                                  .ConsentDetail;

                          print(this.language);
                          print(language);
                          print("========");

                          return ConsentLanguage(
                            language: language ?? "th",
                            isActive: isActive,
                            isFirst: index == 0,
                            isLast: index == (this.pdpaRepository.data?.Setting.AvailableLanguages.length ?? 0) - 1 ,
                            style: style,
                            onClick: (lang) async{
                              try{
                                this.pdpaRepository.toLoadingStatus();
                                this.language = lang;
                                await TimeHelper.sleep();
                                this.pdpaRepository.toLoadedStatus();
                              }catch(e){
                                this.pdpaRepository.alertError(e);
                                this.pdpaRepository.toErrorStatus(e);
                              }
                            },
                          );
                        }),
                      ),
                    ),
                  ],
                ),
              ),
              Container(
                alignment: Alignment.centerLeft,
                padding: EdgeInsets.only(
                  left: isTablet ? 0 : 16,
                  right: isTablet ? 0 : 16,
                ),
                child: Text(
                  isTablet ? "" : this.pdpaRepository.data?.Description ?? "",
                  overflow: TextOverflow.ellipsis,
                  textAlign: TextAlign.start,
                  style: TextStyle(
                    fontSize: 18,
                    fontWeight: FontWeight.bold,
                  ),
                ),
              ),
              Expanded(
                child: Container(
                  margin: EdgeInsets.all(16),
                  decoration: BoxDecoration(
                    border: Border.all(color: Colors.grey.shade400),
                    borderRadius: BorderRadius.circular(8),
                  ),
                  child: ClipRRect(
                    borderRadius: BorderRadius.circular(8),
                    child: ListView(
                      padding: EdgeInsets.zero,
                      children: [
                        ConsentItem(
                          title: "Terms & Conditions",
                          setting: this
                              .pdpaRepository
                              .data
                              ?.Setting
                              .TermsAndConditions,
                          language: this.language,
                          onAllowed: this.onConsentAllowedChanged(
                              ConsentMessageSettingConsentDetails
                                  .terms_and_conditions),
                        ),
                        ConsentItem(
                          title: "Privacy Overview",
                          setting:
                              this.pdpaRepository.data?.Setting.PrivacyOverview,
                          language: this.language,
                          onAllowed: this.onConsentAllowedChanged(
                              ConsentMessageSettingConsentDetails
                                  .privacy_overview),
                        ),
                        ConsentItem(
                          title: "Necessary Cookies",
                          setting: this
                              .pdpaRepository
                              .data
                              ?.Setting
                              .NecessaryCookies,
                          language: this.language,
                          onAllowed: this.onConsentAllowedChanged(
                              ConsentMessageSettingConsentDetails
                                  .necessary_cookies),
                        ),
                        ConsentItem(
                          title: "Preferences Cookies",
                          setting: this
                              .pdpaRepository
                              .data
                              ?.Setting
                              .PreferencesCookies,
                          language: this.language,
                          onAllowed: this.onConsentAllowedChanged(
                              ConsentMessageSettingConsentDetails
                                  .preferences_cookies),
                        ),
                        ConsentItem(
                          title: "Analytics Cookies",
                          setting: this
                              .pdpaRepository
                              .data
                              ?.Setting
                              .AnalyticsCookies,
                          language: this.language,
                          onAllowed: this.onConsentAllowedChanged(
                              ConsentMessageSettingConsentDetails
                                  .analytics_cookies),
                        ),
                        ConsentItem(
                          title: "Marketing Cookies",
                          setting: this
                              .pdpaRepository
                              .data
                              ?.Setting
                              .MarketingCookies,
                          language: this.language,
                          onAllowed: this.onConsentAllowedChanged(
                              ConsentMessageSettingConsentDetails
                                  .marketing_cookies),
                        ),
                        ConsentItem(
                          title: "Social Media Cookies",
                          setting: this
                              .pdpaRepository
                              .data
                              ?.Setting
                              .SocialMediaCookies,
                          language: this.language,
                          onAllowed: this.onConsentAllowedChanged(
                              ConsentMessageSettingConsentDetails
                                  .social_media_cookies),
                        ),
                      ],
                    ),
                  ),
                ),
              ),
              Container(
                padding: EdgeInsets.only(
                    left: 16,
                    right: 16,
                    bottom: 16 + MediaQuery.of(context).padding.bottom),
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.end,
                  children: [
                    PrimaryButton(
                      onClick: () async {
                        await this
                            .pdpaRepository
                            .sendAllowConsentSetting(widget.consentId);
                        Navigator.of(context).pop();
                      },
                      title: "Save Setting",
                      textColor: Colors.white,
                      backgroundColor: this
                          .pdpaRepository
                          .data
                          ?.StyleConfiguration
                          .ConsentDetail
                          .PrimaryColor,
                      margin: EdgeInsets.only(right: 12),
                    ),
                    PrimaryButton(
                      onClick: () async {
                        this.pdpaRepository.allowAllConsentSetting();
                        await this
                            .pdpaRepository
                            .sendAllowConsentSetting(widget.consentId);
                        Navigator.of(context).pop();
                      },
                      title: "Accept All",
                      icon: Icons.check_circle,
                      textColor: Colors.white,
                      backgroundColor: this
                          .pdpaRepository
                          .data
                          ?.StyleConfiguration
                          .ConsentDetail
                          .SecondaryColor,
                    ),
                  ],
                ),
              ),
            ],
          ),
        ],
      ),
    );
  }
}
